/* global Q ZHPromise DeveloperService inject expect */
describe("DeveloperService", function(){

	var developerService;
	var templateManager;
	var $httpBackend;
	var localStorage;

	var LOCALVAR_DEVTOOLS = 'devTools',
		LOCALVAR_CLIENT_EMAIL = 'ds_clientEmail',
		LOCALVAR_ACTIVE_ITEM = 'devToolsActiveItem',
		LOCALVAR_PROJECT_OBJECT = 'ds_projectObject',
		LOGIN_PATH = "json/containers/login.json",
		LOGIN_JSON = {
			"name": "login",
			"id": "01"
		};

	function localStorageMock() {
		var storage = {};
		return {
			setLocalVar: function(key, value) {
				storage[key] = JSON.stringify(value) || '';
			},
			getLocalVar: function(key) {
				var localVar = key in storage ? storage[key] : null;
				return JSON.parse(localVar);
			},
			remove: function(key) {
				delete storage[key];
			}
		};
	}

	beforeEach( function(){
		window.metadataUrl = "./";
		console.error = function() {};
		module('ZeedhiApp');

		inject( function(_restEngine_, _templateManager_, _$httpBackend_){
			var zhPromise = new ZHPromise(Q);
			$httpBackend = _$httpBackend_;
			templateManager = _templateManager_;
			templateManager.projectPromise = zhPromise.when();
			localStorage = localStorageMock();
			developerService = new DeveloperService(_restEngine_, localStorage, templateManager);

		});
	});

	it("SetLocalEmail should set a email in localStorage.", function(){
		var email = "email@teknisa.com";
		developerService.setLocalEmail(email);
		expect(localStorage.getLocalVar(LOCALVAR_CLIENT_EMAIL)).toBe(email);
	});

	it("GetLocalEmail should recover a email in localStorage.", function(){
		var email = "test@teknisa.com";
		developerService.setLocalEmail(email);
		expect(developerService.getLocalEmail()).toBe(email);
	});

	it("SetLocalDevToolsFlag should set a value in LOCALVAR_DEVTOOLS of localStorage.", function(){
		var value = true;
		developerService.setLocalDevToolsFlag(value);
		expect(localStorage.getLocalVar(LOCALVAR_DEVTOOLS)).toBe(value);
	});

	it("SetLocalActiveItem should set an object with a defaul property 'parent' in LOCALVAR_ACTIVE_ITEM of localStorage if hadn't been passed.", function(){
		developerService.setLocalActiveItem("itemName", "path", "mode", undefined, undefined);
		var obj = localStorage.getLocalVar(LOCALVAR_ACTIVE_ITEM);
		expect(obj['parent']).toEqual("");
	});

	it("SetLocalActiveItem should set an objectwith a defaul property 'cursorPosition' in LOCALVAR_ACTIVE_ITEM of localStorage  if hadn't been passed.", function(){
		developerService.setLocalActiveItem("itemName", "path", "mode", undefined, undefined);
		var obj = localStorage.getLocalVar(LOCALVAR_ACTIVE_ITEM);
		expect(obj['cursorPosition']).toEqual({});
	});

	it("SetLocalActiveItem should set an object in LOCALVAR_ACTIVE_ITEM of localStorage .", function(){
		developerService.setLocalActiveItem("itemName", "path", "mode", "parent", "cursorPosition");
		var obj = localStorage.getLocalVar(LOCALVAR_ACTIVE_ITEM);
		expect(obj['name']).toBe("itemName");
		expect(obj['mode']).toBe("mode");
		expect(obj['path']).toBe("path");
		expect(obj['parent']).toBe("parent");
		expect(obj['cursorPosition']).toBe("cursorPosition");
	});

	it("GetLocalDevToolsFlag should return a value in LOCALVAR_DEVTOOLS of localStorage.", function(){
		var value = false;
		developerService.setLocalDevToolsFlag(value);
		expect(developerService.getLocalDevToolsFlag()).toBe(value);
	});

	it("GetLocalActiveItem should return a value in LOCALVAR_ACTIVE_ITEM of localStorage.", function(){
		developerService.setLocalActiveItem("itemName", "path", "mode", "parent", "cursorPosition");
		var obj = developerService.getLocalActiveItem();
		expect(obj['name']).toBe("itemName");
		expect(obj['mode']).toBe("mode");
		expect(obj['path']).toBe("path");
		expect(obj['parent']).toBe("parent");
		expect(obj['cursorPosition']).toBe("cursorPosition");
	});

	it("ClearActiveItem should return null in LOCALVAR_ACTIVE_ITEM of localStorage.", function(){
		developerService.setLocalActiveItem("itemName", "path", "mode", "parent", "cursorPosition");
		var obj = developerService.getLocalActiveItem();
		expect(obj['name']).toBe("itemName");
		obj = developerService.clearActiveItem();
		developerService.getLocalActiveItem();
		expect(obj).toBe(undefined);
	});

	it("SetProject should set properties of project in localStorage.", function(){
		templateManager.project.id = '123';
		templateManager.project.label = 'label of project';
		developerService.setProject();
		var localProject = localStorage.getLocalVar(LOCALVAR_PROJECT_OBJECT);
		expect(localProject['id']).toBe("123");
		expect(localProject['name']).toBe("label of project");
	});

	it("GetProject should return a ds_projectObject in localStorage.", function(){
		templateManager.project.id = '123';
		templateManager.project.label = 'label of project';
		developerService.setProject();
		var localProject = developerService.getProject();
		expect(localProject['id']).toBe("123");
		expect(localProject['name']).toBe("label of project");
	});

	it("InDeveloperModePromise when 'MainWindow.showDevtoolsInSameWindow' is undefined should set 'window.showDevtoolsInSameWindow' with a value of 'templateManager.project.showDevtoolsInSameWindow'.", function(done){
		window.showDevtoolsInSameWindow = undefined;
		templateManager.project.showDevtoolsInSameWindow = true;
		developerService.inDeveloperModePromise.then(function(){
			expect(window.showDevtoolsInSameWindow).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'MainWindow.showDevtoolsInSameWindow' is defined shouldn't set 'MainWindow.showDevtoolsInSameWindow'", function(done){
		window.showDevtoolsInSameWindow = false;
		developerService.inDeveloperModePromise.then(function(){
			expect(window.showDevtoolsInSameWindow).toEqual(false);
			done();
		});
	});

	it("InDeveloperModePromise when 'window.inCustomizationMode' is undefined should copy in 'window.inCustomizationMode' and 'templateManager.inCustomizationMode " +
	"the value of 'templateManager.project.inCustomizationMode'.", function(done){
		window.inCustomizationMode = undefined;
		templateManager.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = undefined;
		templateManager.project.inCustomizationMode = true;
		developerService.inDeveloperModePromise.then(function(){
			expect(window.inCustomizationMode).toEqual(true);
			expect(templateManager.inCustomizationMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise should set 'templateManager.project.inCustomizationMode' and 'templateManager.inCustomizationMode' of project with true, when his value in window is true.", function(done){
		window.inCustomizationMode = true;
		templateManager.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(){
			expect(templateManager.project.inCustomizationMode).toEqual(true);
			expect(templateManager.inCustomizationMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'window.inCustomizationMode' is true should return a object with a property 'inCustomizationMode' in true.", function(done){
		window.inCustomizationMode = true;
		templateManager.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(response.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'window.inCustomizationMode' is true should return a object with a property 'inCustomizationMode' in true.", function(done){
		window.inCustomizationMode = true;
		templateManager.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(response.inCustomizationMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'window.inCustomizationMode' is true should set with true 'window.inDeveloperMode'.", function(done){
		window.inCustomizationMode = true;
		window.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(window.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'window.inCustomizationMode' is true should set with true 'templateManager.inDeveloperMode'.", function(done){
		window.inCustomizationMode = true;
		templateManager.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(templateManager.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'window.inCustomizationMode' is true should set with true 'templateManager.project.inDeveloperMode'.", function(done){
		window.inCustomizationMode = true;
		templateManager.project.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(templateManager.project.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.project.inDeveloperMode' is true should return a object with the property 'inDeveloperMode' with true.", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = true;
		developerService.inDeveloperModePromise.then(function(response){
			expect(response.inDeveloperMode).toEqual(true);
			expect(response.inCustomizationMode).toEqual(false);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.project.inDeveloperMode' is true should set 'window.inDeveloperMode' with true.", function(done){
		window.inCustomizationMode = false;
		window.inDeveloperMode = false;
		templateManager.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = true;
		developerService.inDeveloperModePromise.then(function(response){
			expect(window.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.project.inDeveloperMode' is true should set 'templateManager.inDeveloperMode' with true.", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = true;
		developerService.inDeveloperModePromise.then(function(response){
			expect(templateManager.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.project.inDeveloperMode' is true should stay 'templateManager.project.inDeveloperMode' in true.", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = true;
		developerService.inDeveloperModePromise.then(function(response){
			expect(templateManager.project.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.inDeveloperMode' is true should set 'window.inDeveloperMode' with true. ", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = true;
		templateManager.project.inDeveloperMode = undefined;
		window.inDeveloperMod = false;
		developerService.inDeveloperModePromise.then(function(response){
			expect(window.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.inDeveloperMode' is true should stay 'templateManager.inDeveloperMode' in true. ", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = true;
		templateManager.project.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(templateManager.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.inDeveloperMode' is true should set 'templateManager.project.inDeveloperMode' with true. ", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = true;
		templateManager.project.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = false;
		developerService.inDeveloperModePromise.then(function(response){
			expect(templateManager.project.inDeveloperMode).toEqual(true);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.inDeveloperMode' is true should return an object with the preperty 'inDeveloperMode' in true. ", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = true;
		templateManager.project.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(response.inDeveloperMode).toEqual(true);
			expect(response.inCustomizationMode).toEqual(false);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.inDeveloperMode' and 'window.inCustomizationMode' are false should set 'window.inDeveloperMode' with false. ", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = false;
		templateManager.project.inDeveloperMode = undefined;
		window.inDeveloperMod = false;
		developerService.inDeveloperModePromise.then(function(response){
			expect(window.inDeveloperMode).toEqual(false);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.inDeveloperMode' and 'window.inCustomizationMode' are false should stay 'templateManager.inDeveloperMode' in false. ", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = false;
		templateManager.project.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(templateManager.inDeveloperMode).toEqual(false);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.inDeveloperMode' and 'window.inCustomizationMode' are false should set 'templateManager.project.inDeveloperMode' with false. ", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = false;
		templateManager.project.inDeveloperMode = undefined;
		templateManager.project.inDeveloperMode = false;
		developerService.inDeveloperModePromise.then(function(response){
			expect(templateManager.project.inDeveloperMode).toEqual(false);
			done();
		});
	});

	it("InDeveloperModePromise when 'templateManager.inDeveloperMode' and 'window.inCustomizationMode' are false" +
	"should return an object with the properties 'inDeveloperMode' and 'inCustomizationMode' in false. ", function(done){
		window.inCustomizationMode = false;
		templateManager.inDeveloperMode = false;
		templateManager.project.inDeveloperMode = undefined;
		developerService.inDeveloperModePromise.then(function(response){
			expect(response.inDeveloperMode).toEqual(false);
			expect(response.inCustomizationMode).toEqual(false);
			done();
		});
	});

	it("RequestMetadataDeveloper should be return the content of path.", function(done){
		$httpBackend.expectGET('./' + LOGIN_PATH).respond(LOGIN_JSON);
		developerService.requestMetadataDeveloper(LOGIN_PATH).then(function(response){
			var data = response.data;
			expect(data.name).toEqual(LOGIN_JSON.name);
			expect(data.id).toEqual(LOGIN_JSON.id);
			done();
		});
		$httpBackend.flush();
	});

	it("RequestMetadataDeveloper should be return the content of path, after set path via catch.", function(done){
		templateManager.getMappedPackages = function(){
			return { "default": false};
		};
		$httpBackend.expectGET('./' + LOGIN_PATH).respond(LOGIN_JSON);
		developerService.requestMetadataDeveloper(LOGIN_PATH).then(function(response){
			var data = response.data;
			expect(data.name).toEqual(LOGIN_JSON.name);
			expect(data.id).toEqual(LOGIN_JSON.id);
			done();
		});
		$httpBackend.flush();
	});
});

